/**
 * @module dixim
 * @submodule download
 */
dixim.namespace('dixim.download');

/**
 * @module download
 * @submodule download-controller
 */

(function() {
var DownloadController;
var prototype;
var superConstructor;
var DownloadView = dixim.contents.DownloadView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var UiFunctionManager = dixim.ctt.util.UiFunctionManager;
var ApiClientTransfer = dixim.common.ApiClientTransfer;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var DownloadWizardDialogController = dixim.download.DownloadWizardDialogController;
var SettingMenuDialogController = dixim.download.SettingMenuDialogController;
var Locker = dixim.ctt.util.Locker;
var Utils = dixim.ctt.util.Utils;

/**
 * @class DownloadController
 * @constructor
 * @param {Object} configs
 */
DownloadController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(DownloadController, EventTarget);
dixim.download.DownloadController = DownloadController;
prototype = DownloadController.prototype;
superConstructor = DownloadController.superConstructor;

UiFunctionManager.set('downloadMedia');

var SELECT = 'select';
var SELECT_CONTENT = 'selectContent';
var SELECT_SERVER = 'selectServer';
var DOWNLOAD = 'download';
var REFRESH = 'refresh';
var SETTING = 'setting';
var PREV = 'prev';
var FINISH = 'finish';
var CLOSE = 'close';

var BROWSING_STATUS = {
    STOPPED: 0,
    STOPPING: 1,
    BROWSING: 2,
    PAUSING: 3,
    PAUSED: 4,
};

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'download'].join('');

    this._model = new ApiClientTransfer({isCancelAll: true});
    this._model.addEventListener('fetchDnDmsList', this._successFetchDnDmsList, this);
    this._model.addEventListener('fetchDnDmsListError', this._errorFetchDnDmsList, this);
    this._model.addEventListener('dnBrowse', this._successDnBrowse, this);
    this._model.addEventListener('dnBrowseError', this._errorDnBrowse, this);

    this._view = new DownloadView(configs);
    this._view.addEventListener(SELECT, this._clickSelect, this);
    this._view.addEventListener(SELECT_SERVER, this._clickSelectServer, this);
    this._view.addEventListener(SELECT_CONTENT, this._clickSelectContent, this);
    this._view.addEventListener(DOWNLOAD, this._clickDownload, this);
    this._view.addEventListener(REFRESH, this._clickRefresh, this);
    this._view.addEventListener(SETTING, this._clickSetting, this);
    this._view.addEventListener(PREV, this._clickPrev, this);

    this._locker = new Locker();
    this._parentContext = configs.parentContext;
    this._serverInfo = null;
    this._browsingStatus = BROWSING_STATUS.STOPPED;
    this._browseRequestData = {};
    this._browsingStack = [];
    this._isIE = (Utils.getBrowser().indexOf('ie') != -1);
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._fetchDnDmsList();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._fetchDnDmsList = function() {
    var query = {id: 'server'};

    this._view.startLoading();
    this._browseRequestData = query;
    this._browsingStatus = BROWSING_STATUS.BROWSING;
    this._model.fetchDnDmsList();
};

prototype._successFetchDnDmsList = function(e) {
    var results = e.args.results;
    var resultsLen = results.length;

    this._view.stopLoading();
    if (resultsLen === 0) {
        this._view.noRow(true);
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        return;
    }

    for (var i = 0; i < resultsLen; i += 1) {
        this._view.addRow(results[i]);
    }
    this._browsingStatus = BROWSING_STATUS.STOPPED;
};

prototype._errorFetchDnDmsList = function() {
    this._view.stopLoading();
    this._view.errorRow(true);
};

prototype._viewContents = function(results) {
    var resultsLen = results.length;

    for (var i = 0; i < resultsLen; i += 1) {
        this._view.addContent(results[i]);
    }
    this._view.checkActionBarStatus();
};

prototype._successDnBrowse = function(e) {
    var totalReturned = e.args.numberReturned;
    var totalMatches = e.args.totalMatches;
    var startingIndex = e.args.startingIndex;
    var results = e.args.results;
    var isStopping = (this._browsingStatus === BROWSING_STATUS.STOPPING);
    var isPausing = (this._browsingStatus === BROWSING_STATUS.PAUSING);
    var isEndBrowsing = ((totalReturned + startingIndex) === totalMatches);
    var isMatchId = true;

    if (results.length !== 0) isMatchId = (this._browseRequestData.id === results[0].parentId);

    if (totalMatches === 0) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        this._view.noRow(false);
        this._view.stopLoading();
        return;
    }
    if (isStopping || (!isMatchId)) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        this._view.stopLoading();
        return;
    }
    if (isPausing) {
        this._browsingStatus = BROWSING_STATUS.PAUSED;
        this._view.stopLoading();
        return;
    }

    this._viewContents(results);

    if (isEndBrowsing) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        this._view.stopLoading();
        return;
    }

    totalReturned += this._browseRequestData.starting_index;

    if (this._isIE) {
        setTimeout(dixim.proxy(function() {
            this._browseRequest(this._browseRequestData.id, totalReturned)
        }, this), 100);
    } else {
        this._browseRequest(this._browseRequestData.id, totalReturned);
    }
};

prototype._errorDnBrowse = function() {
    this._errorDialog(_('download_fetch_contents_error'));
    this._browsingStatus = BROWSING_STATUS.STOPPED;
    this._view.stopLoading();
    this._clickPrev();
};

prototype._browse = function(id) {
    this._view.startLoading();
    this._browsingStatus = BROWSING_STATUS.BROWSING;
    this._browseRequest(id);
};

prototype._restartBrowse = function() {
    if (this._browsingStatus !== BROWSING_STATUS.PAUSED) return;

    this._view.startLoading();
    this._browsingStatus = BROWSING_STATUS.BROWSING;
    this._browseRequest(this._browseRequestData.id, this._browseRequestData.starting_index);
};

prototype._browseRequest = function(id, startingIndex) {
    var query = {};

    query.udn = this._serverInfo.udn;
    query.id = id;
    query.starting_index = startingIndex || 0;
    query.requested_count = 20;

    this._browseRequestData = query;
    this._model.dnBrowse(query);
};

prototype._browseAfterStop = function(id) {
    if (this._browsingStatus === BROWSING_STATUS.BROWSING) {
        this._browsingStatus = BROWSING_STATUS.STOPPING;
    }

    if (this._browsingStatus !== BROWSING_STATUS.STOPPED) {
        setTimeout(dixim.proxy(function() {
            this._browseAfterStop(id)
        }, this), 100);
    } else {
        this._view.cleanContent();
        this._browse(id);
    }
};

prototype._clickSelectServer = function(e) {
    this._serverInfo = e.args;
    this._browsingStack.push('server');
    this._browseAfterStop('0');
    this._view.setContentName(this._serverInfo.friendlyName);
};

prototype._clickSelectContent = function(e) {
    var data = e.args;

    this._browsingStack.push(data.parentId);
    this._browseAfterStop(data.id);
    this._view.setContentName(data.title);
};

prototype._clickRefresh = function() {
    this._update();
};

prototype._clickSetting = function() {
    var controller;

    controller = new SettingMenuDialogController();
    controller.start();
};

prototype._clickSelect = function() {
    if (this._browsingStatus === BROWSING_STATUS.BROWSING) {
        this._errorDialog(_('download_fetching_error', _('action_bar_select')));
    } else {
        this._view.switchSelect();
    }
};

prototype._clickDownload = function(e) {
    var controller;
    var query = {};
    var dataList = e.args.dataList;

    if (this._locker.isLock()) return;
    this._locker.lock();

    if (dataList.length === 0) {
        this._errorDialog(_('download_error_operation_description'));
        return;
    }

    query.serverInfo = this._serverInfo;
    query.dataList = dataList;
    controller = new DownloadWizardDialogController(query);
    controller.addEventListener(FINISH, this._update, this);
    controller.addEventListener(FINISH, this._finishDialog, this);
    if (this._browsingStatus === BROWSING_STATUS.BROWSING) {
        this._browsingStatus = BROWSING_STATUS.PAUSING;
        controller.addEventListener(CLOSE, this._restartBrowse, this);
    }
    controller.addEventListener(CLOSE, this._finishDialog, this);
    controller.start();
};

prototype._clickPrev = function() {
    var id = this._browsingStack.pop();

    this._view.unsetContentName();
    if (typeof id === 'undefined') {
        this.stop();
        this._parentContext.start();
    } else if (id === 'server') {
        this._view.cleanContent();
        this._fetchDnDmsList();
    } else {
        this._browseAfterStop(id);
    }
};

prototype._update = function() {
    var id = this._browseRequestData.id;

    if (id === 'server') {
        this._view.cleanContent();
        this._fetchDnDmsList();
    } else {
        this._browseAfterStop(id);
    }
};

prototype._finishDialog = function() {
    this._locker.unlock();
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = _('common_menu_title_download');
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
