/**
 * @module dixim
 * @submodule common
 */
dixim.namespace('dixim.common');

/**
 * @module common
 * @submodule api-client-server
 */
(function() {
var ApiClientServer;
var prototype;
var suprtConstructor;
var ApiClient = dixim.common.ApiClient;
var Configs = dixim.ctt.Configs;

/**
 * Provides interfaces to setting server API.
 *
 * This API client supports two type of event handling method,
 * callback and event-subscription.
 *
 * The callback method can be specified as the parameter of each method.
 * (Such as request.success, request.error or request.complete.)
 * When the callback is called, an Event instance which is related to
 * the triggered method is passed to the callback.
 *
 * This API client is also an EventTarget, you can subscribe an event
 * by addEventListener().
 *
 * @class ApiClientServer
 * @constructor
 * @extends EventTarget
 * @param {Object} [configs]
 */
ApiClientServer = function(configs) {
    ApiClient.call(this, configs);
};

dixim.inherit(ApiClientServer, ApiClient);
dixim.common.ApiClientServer = ApiClientServer;
prototype = ApiClientServer.prototype;
superConstructor = ApiClientServer.suprtConstructor;

/**
 * Fetches friendly name.
 *
 * @method fetchFriendlyName
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchFriendlyName = function(request) {
    return this.doSimpleFetch.call(this, 'fetchFriendlyName', [Configs.get('apiDcs'), '/friendly_name/get'].join(''), request);
};

/**
 * Change friendly name.
 *
 * @method changeFriendlyName
 * @param {Object} request
 * @param {String} [request.friendlyName]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.changeFriendlyName = function(request) {
    var query = {};

    query.friendlyName = request.friendlyName;

    requesy = request || {};
    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'changeFriendlyName', [Configs.get('apiDcs'), '/friendly_name/set'].join(''), query, request);
};

/**
 * Reset friendly name.
 *
 * @method resetFriendlyName
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.resetFriendlyName = function(request) {
    var query = {};

    request = request || {};
    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'resetFriendlyName', [Configs.get('apiDcs'), '/friendly_name/reset'].join(''), query, request);
};

/**
 * Fetch allow list.
 *
 * @method fetchAllowList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchAllowList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchAllowList', [Configs.get('apiAcs'), '/GetAllowList'].join(''), request, 'POST');
};

/**
 * Fetch deny list.
 *
 * @method fetchDenyList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchDenyList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchDenyList', [Configs.get('apiAcs'), '/GetDenyList'].join(''), request, 'POST');
};

/**
 * Set permission.
 *
 * @method setPermission
 * @param {Object} request
 * @param {Array} [request.MacList]
 * @param {String} [request.Permission]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setPermission = function(request) {
    var query = {};

    query.MacList = request.MacList.join(',');
    query.Permission = request.Permission;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'setPermission', [Configs.get('apiAcs'), '/SetPermission'].join(''), query, request, 'POST');
};

/**
 * remove entries.
 *
 * @method removeEntries
 * @param {Object} request
 * @param {Array} [request.Entries]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removeEntries = function(request) {
    var query = {};

    query.Entries = request.Entries.join(',');

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removeEntries', [Configs.get('apiAcs'), '/RemoveEntries'].join(''), query, request, 'POST');
};

/**
 * get default Permission.
 *
 * @method getDefaultPermission
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.getDefaultPermission = function(request) {
    return this.doSimpleFetch.call(this, 'getDefaultPermission', [Configs.get('apiAcs'), '/GetDefaultPermission'].join(''), request);
};

/**
 * set default Permission.
 *
 * @method setDefaultPermission
 * @param {Object} request
 * @param {String} [request.DefaultPermission]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setDefaultPermission = function(request) {
    var query = {};

    query.DefaultPermission = request.DefaultPermission;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'setDefaultPermission', [Configs.get('apiAcs'), '/SetDefaultPermission'].join(''), query, request);
};

/**
 * Fetches dms status.
 *
 * @method fetchDmsStatus
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchDmsStatus = function(request) {
    return this.doSimpleFetch.call(this, 'fetchDmsStatus', [Configs.get('apiDeviceManager'), '/status/get'].join(''), request);
};

/**
 * Change dms status.
 *
 * @method fetchDmsStatus
 * @param {Object} request
 * @param {String} [request.status]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.changeDmsStatus = function(request) {
    var query = {};

    query.status = request.status;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'changeDmsStatus', [Configs.get('apiDeviceManager'), '/status/mod'].join(''), query, request);
};

/**
 * Sync shared folder.
 *
 * @method syncSharedFolder
 * @param {Object} request
 * @param {String} [request.sharedFolder]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.syncSharedFolder = function(request) {
    var query = {};

    query.sharedFolder = request.sharedFolder;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'syncSharedFolder', [Configs.get('apiCds'), '/shared_folder/sync'].join(''), query, request);
};

/**
 * Fetches shared folder.
 *
 * @method fetchSharedFolder
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchSharedFolder = function(request) {
    return this.doSimpleFetch.call(this, 'fetchSharedFolder', [Configs.get('apiCds'), '/shared_folder/get'].join(''), request);
};

/**
 * Fetches shared folder.
 *
 * @method fetchSharedFolder2
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchSharedFolder2 = function(request) {
    return this.doSimpleFetch.call(this, 'fetchSharedFolder2', [Configs.get('apiCds'), '/shared_folder/get2'].join(''), request);
};

/**
 * Set shared folder.
 *
 * @method setSharedFolder
 * @param {Object} request
 * @param {String} [request.sharedFolder]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setSharedFolder = function(request) {
    var query = {};

    query.sharedFolder = request.sharedFolder;
    query.mediaType = request.mediaType;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'setSharedFolder', [Configs.get('apiCds'), '/shared_folder/add'].join(''), query, request);
};

/**
 * Remove shared folder.
 *
 * @method removeSharedFolder
 * @param {Object} request
 * @param {String} [request.sharedFolder]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removeSharedFolder = function(request) {
    var query = {};

    query.sharedFolder = request.sharedFolder;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removeSharedFolder', [Configs.get('apiCds'), '/shared_folder/del'].join(''), query, request);
};

/**
 * Fetch shared type.
 *
 * @method fetchSharedType
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchSharedType = function(request) {
    return this.doSimpleFetch.call(this, 'fetchSharedType', [Configs.get('apiCds'), '/shared_type/get'].join(''), request);
};

/**
 * Set shared type.
 *
 * @method setSharedType
 * @param {Object} request
 * @param {Array} [request.extensions]
 * @param {Array} [request.mimeTypes]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setSharedType = function(request) {
    var query = {};

    query.extensions = request.extensions.join(',');
    query.mimeTypes = request.mimeTypes.join(',');

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'setSharedType', [Configs.get('apiCds'), '/shared_type/set'].join(''), query, request, 'POST');
};

/**
 * Remove shared type.
 *
 * @method removeSharedType
 * @param {Object} request
 * @param {Array} [request.extensions]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removeSharedType = function(request) {
    var query = {};

    query.extensions = request.extensions.join(',');

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removeSharedType', [Configs.get('apiCds'), '/shared_type/del'].join(''), query, request);
};

/**
 * Remove shared type in more detail.
 *
 * @method removeSharedType2
 * @param {Object} request
 * @param {Array} [request.extensions]
 * @param {Array} [request.mimeTypes]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removeSharedType2 = function(request) {
    var query = {};

    query.extensions = request.extensions.join(',');
    query.mimeTypes = request.mimeTypes.join(',');

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removeSharedType2', [Configs.get('apiCds'), '/shared_type/del2'].join(''), query, request);
};

/**
 * Reset shared type.
 *
 * @method resetSharedType
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.resetSharedType = function(request) {
    var query = {};

    request = request || {};
    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'resetSharedType', [Configs.get('apiCds'), '/shared_type/reset'].join(''), query, request);
};

/**
 * Fetch pairing devices.
 *
 * @method fetchPairingDevice
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchPairingDevice = function(request) {
    return this.doSimpleFetch.call(this, 'fetchPairingDevice', [Configs.get('apiDtcpPlus'), '/device/get'].join(''), request, 'POST');
};

/**
 * Remove pairing devices.
 *
 * @method removePairingDevice
 * @param {Object} request
 * @param {String} [request.deviceId]
 * @param {String} [request.dtlaIdHash]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removePairingDevice = function(request) {
    var query = {};

    query.deviceId = request.deviceId;
    query.dtlaIdHash = request.dtlaIdHash;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removePairingDevice', [Configs.get('apiDtcpPlus'), '/device/del'].join(''), query, request);
};

/**
 * Fetch transfer status.
 *
 * @method fetchTransferStatus
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchTransferStatus = function(request) {
    return this.doSimpleFetch.call(this, 'fetchTransferStatus', [Configs.get('apiMtms'), '/status/get'].join(''), request, 'POST');
};

/**
 * Fetches dtcp-ip activation status.
 *
 * @method fetchDas
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchDas = function(request) {
    return this.doSimpleFetch.call(this, 'fetchDas', [Configs.get('apiDas'), '/status/get'].join(''), request);
};

/**
 * Activate dtcp-ip auth.
 *
 * @method activateDas
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.activateDas = function(request) {
    return this.doSimpleFetch.call(this, 'activateDas', [Configs.get('apiDas'), '/activate'].join(''), request);
};

/**
 * Fetch record schedule.
 *
 * @method fetchRecordSchedule
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchRecordSchedule = function(request) {
    return this.doSimpleFetch.call(this, 'fetchRecordSchedule', [Configs.get('apiXsrs'), '/record_schedule/get'].join(''), request);
};

/**
 * Browse file system.
 *
 * @method browseFileSystem
 * @param {Object} request
 * @param {String} [request.path]
 * @param {Integer} [request.enableHiddenDir]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.browseFileSystem = function(request) {
    var query = {};

    query.path = request.path;
    query.enableHiddenDir = request.enableHiddenDir;

    return this.doSimpleMod.call(this, 'browseFileSystem', [Configs.get('apiFs'), '/browse'].join(''), query, request);
};

/**
 * Fetch language.
 *
 * @method fetchLanguage
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchLanguage = function(request) {
    return this.doSimpleFetch.call(this, 'fetchLanguage', [Configs.get('apiCds'), '/language/get'].join(''), request);
};

/**
 * Set language.
 *
 * @method setLanguage
 * @param {Object} request
 * @param {String} [request.language]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setLanguage = function(request) {
    var query = {};

    query.language = request.language;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'setLanguage', [Configs.get('apiCds'), '/language/set'].join(''), query, request);
};

}());
