/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule extensions-edit-dialog-controller
 */

(function() {
var ExtensionsEditDialogController;
var prototype;
var superConstructor;
var ExtensionsEditDialogView = dixim.server.ExtensionsEditDialogView;
var EventTarget = dixim.event.EventTarget;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var ApiClientServer = dixim.common.ApiClientServer;

/**
 * @class ExtensionsEditDialogController
 * @constructor
 * @param {Object} configs
 */
ExtensionsEditDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ExtensionsEditDialogController, EventTarget);
dixim.server.ExtensionsEditDialogController = ExtensionsEditDialogController;
prototype = ExtensionsEditDialogController.prototype;
superConstructor = ExtensionsEditDialogController.superConstructor;

var OK = 'ok';
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'extensions-edit-dialog'].join('');

    this._model = new ApiClientServer();
    this._model.addEventListener('setSharedType', this._successSetSharedType, this);
    this._model.addEventListener('setSharedTypeError', this._errorSetSharedType, this);
    this._model.addEventListener('removeSharedType2', this._successRemoveSharedType, this);
    this._model.addEventListener('removeSharedType2Error', this._errorRemoveSharedType, this);

    this._view = new ExtensionsEditDialogView(configs);
    this._view.addEventListener(OK, this._clickOk, this);

    this._itemData = configs.itemData || null;
    this._sharedTypes = configs.sharedTypes || [];
    this._modifiedData;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
    this.dispatchEvent(new Event(CLOSE));
};

prototype._checkJapaneseText = function(data) {
    var extensionLen = data.extension.length;
    var mimetypeLen = data.mimetype.length;

    for (var i = 0; i < extensionLen; i += 1) {
        if (data.extension.charCodeAt(i) > 255) {
            return false;
        }
    }
    for (var j = 0; j < mimetypeLen; j += 1) {
        if (data.mimetype.charCodeAt(j) > 255) {
            return false;
        }
    }
    return true;
};

prototype._isDuplicateSharedType = function(data) {
    var sharedTypes = this._sharedTypes;
    var isMatchExtension;
    var isMatchMimetype;

    for (var i = 0, len = sharedTypes.length; i < len; i += 1) {
        isMatchExtension = (sharedTypes[i].extension.toLowerCase() === data.extension.toLowerCase());
        isMatchMimetype = (sharedTypes[i].mimeType.toLowerCase() === data.mimetype.toLowerCase());
        if (isMatchExtension && isMatchMimetype) return true;
    }
    return false;
};

prototype._editSharedType = function(data) {
    var query = {};
    var isMatchExtension = ((this._itemData !== null) && (this._itemData.extension === data.extension));
    var isMatchMimetype = ((this._itemData !== null) && (this._itemData.mimeType === data.mimetype));

    if (!!!this._checkJapaneseText(data)) {
        this.stop();
        this._errorDialog(_('server_extension_dialog_japanese_error'));
        return;
    }
    if (this._itemData === null) {
        if (this._isDuplicateSharedType(data)) {
            this.stop();
            this._errorDialog(_('server_extension_dialog_duplicate_error'));
            return;
        } else {
            this._modifiedData = data;
            this._setSharedType();
        }
    } else {
        if (isMatchExtension && isMatchMimetype) {
            this.stop();
            return;
        }
        if (this._isDuplicateSharedType(data)) {
            this.stop();
            this._errorDialog(_('server_extension_dialog_duplicate_error'));
            return;
        }
        this._modifiedData = data;
        query.extensions = [this._itemData.extension];
        query.mimeTypes = [this._itemData.mimeType];
        this._model.removeSharedType2(query);
    }
};

prototype._successRemoveSharedType = function() {
    this._setSharedType();
};

prototype._errorRemoveSharedType = function() {
    this.stop();
    this._errorDialog(_('server_extension_dialog_could_not_edit'));
};

prototype._setSharedType = function() {
    var query = {};

    query.extensions = [this._modifiedData.extension];
    query.mimeTypes = [this._modifiedData.mimetype];
    this._model.setSharedType(query);
};

prototype._successSetSharedType = function() {
    this.stop();
};

prototype._errorSetSharedType = function() {
    var query = {};
    var description = (this._itemData === null) ? _('server_extension_dialog_could_not_add') : _('server_extension_dialog_could_not_edit');

    this.stop();
    this._errorDialog(description);
};

prototype._clickOk = function(e) {
    this._view.startIndicator();
    this._editSharedType(e.args);
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = (this._itemData === null) ? _('server_extension_dialog_add_title') : _('server_extension_dialog_edit_title');
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
